@@ -1,5 +1,5 @@
 /*
-** Copyright 2000-2016 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2018 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -26,6 +26,7 @@
 #include	"courierauth.h"
 #include	"courierauthstaticlist.h"
 #include        "libhmac/hmac.h"
+#include	"authdaemond.h"
 #include	<ltdl.h>
 
 
@@ -45,6 +46,8 @@ struct authstaticinfolist {
 
 static struct authstaticinfolist *modulelist=NULL;
 
+char tcpremoteip[40];
+
 static int mksocket()
 {
 int	fd=socket(PF_UNIX, SOCK_STREAM, 0);
@@ -52,14 +55,15 @@ struct	sockaddr_un skun;
 
 	if (fd < 0)	return (-1);
 	skun.sun_family=AF_UNIX;
-	strcpy(skun.sun_path, AUTHDAEMONSOCK);
-	strcat(skun.sun_path, ".tmp");
+	memcpy(skun.sun_path, AUTHDAEMONSOCK, sizeof(AUTHDAEMONSOCK)-1);
+	memcpy(&skun.sun_path[0]+sizeof(AUTHDAEMONSOCK)-1, ".tmp", 5);
 	unlink(skun.sun_path);
 	if (bind(fd, (const struct sockaddr *)&skun, sizeof(skun)) ||
-		listen(fd, SOMAXCONN) ||
-		chmod(skun.sun_path, 0777) ||
-		rename(skun.sun_path, AUTHDAEMONSOCK) ||
-		fcntl(fd, F_SETFL, O_NONBLOCK) < 0)
+	    listen(fd, SOMAXCONN) ||
+	    chmod(skun.sun_path, 0777) ||
+	    rename(skun.sun_path, AUTHDAEMONSOCK) ||
+	    fcntl(fd, F_SETFD, FD_CLOEXEC) < 0 ||
+	    fcntl(fd, F_SETFL, O_NONBLOCK) < 0)
 	{
 		perror(AUTHDAEMONSOCK);
 		close(fd);
@@ -111,10 +115,12 @@ static int initmodules(const char *p)
 
 		strcpy(buf, "lib");
 		strncat(buf, p, i>40 ? 40:i);
+		strcpy(buf2, buf);
+		strcat(buf2, COURIERDLEXT);
 
-		fprintf(stderr, "INFO: Installing %s\n", buf);
+		fprintf(stderr, "INFO: Installing %s\n", buf2);
 		p += i;
-		h=lt_dlopenext(buf);
+		h=lt_dlopen(buf2);
 
 		if (h == NULL)
 		{
@@ -122,7 +128,8 @@ static int initmodules(const char *p)
 			continue;
 		}
 
-		sprintf(buf2, "courier_%s_init", buf+3);
+		buf2[snprintf(buf2, sizeof(buf2)-1, "courier_%s_init",
+			      buf+3)]=0;
 
 		pt=lt_dlsym(h, buf2);
 		if (pt == NULL)
@@ -793,22 +800,41 @@ int	i, ch;
 char	*p;
 
 	readleft=0;
-	for (i=0; (ch=getauthc(fd)) != '\n'; i++)
-	{
-		if (ch < 0 || i >= sizeof(buf)-2)
-			return;
-		buf[i]=ch;
-	}
-	buf[i]=0;
+	tcpremoteip[0]=0;
 
-	for (p=buf; *p; p++)
+	while (1)
 	{
+		for (i=0; (ch=getauthc(fd)) != '\n'; i++)
+		{
+			if (ch < 0 || i >= sizeof(buf)-2)
+				return;
+			buf[i]=ch;
+		}
+		buf[i]=0;
+
+		for (p=buf; *p; p++)
+		{
+			if (*p == ' ' || *p == '=')
+				break;
+		}
+
+		if (!*p)
+			break;
+
 		if (*p == ' ')
 		{
 			*p++=0;
 			while (*p == ' ')	++p;
 			break;
 		}
+
+		*p++=0;
+
+		if (strcmp(buf, "TCPREMOTEIP") == 0)
+		{
+			tcpremoteip[sizeof(tcpremoteip)-1]=0;
+			strncpy(tcpremoteip, p, sizeof(tcpremoteip)-1);
+		}
 	}
 
 	if (strcmp(buf, "PRE") == 0)
@@ -1024,7 +1050,8 @@ int start()
 		saddr_len=sizeof(saddr);
 		if ((fd=accept(s, &saddr, &saddr_len)) < 0)
 			continue;
-		if (fcntl(fd, F_SETFL, 0) < 0)
+		if (fcntl(fd, F_SETFL, 0) < 0 ||
+		    fcntl(fd, F_SETFD, FD_CLOEXEC) < 0)
 		{
 			perror("CRIT: fcntl() failed");
 		}
