@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.
+** Copyright 1998 - 2007 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -13,7 +13,6 @@
 #include	<stdio.h>
 #include	<errno.h>
 
-static const char rcsid[]="$Id: userdb2.c,v 1.5 2006/10/28 19:22:52 mrsam Exp $";
 
 extern int userdb_debug_level;
 
@@ -45,7 +44,11 @@ size_t	l;
 	}
 
 	p=malloc(l+1);
-	if (!p)	return (0);
+	if (!p)
+	{
+		free(q);
+		return (0);
+	}
 
 	if (l)	memcpy(p, q, l);
 	free(q);
